/**
 * DevExtreme (cjs/ui/scheduler/appointments.layout_manager.js)
 * Version: 21.2.0 (build 21230-1556)
 * Build date: Wed Aug 18 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.default = void 0;
var _common = require("../../core/utils/common");
var _instanceFactory = require("./instanceFactory");
var _viewModelGenerator = require("./appointments/viewModelGenerator");

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}

function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) {
            descriptor.writable = true
        }
        Object.defineProperty(target, descriptor.key, descriptor)
    }
}

function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) {
        _defineProperties(Constructor.prototype, protoProps)
    }
    if (staticProps) {
        _defineProperties(Constructor, staticProps)
    }
    return Constructor
}
var AppointmentLayoutManager = function() {
    function AppointmentLayoutManager(instance) {
        this.instance = instance;
        this.appointmentViewModel = new _viewModelGenerator.AppointmentViewModel
    }
    var _proto = AppointmentLayoutManager.prototype;
    _proto.getCellDimensions = function(options) {
        if (this.instance._workSpace) {
            return {
                width: this.instance._workSpace.getCellWidth(),
                height: this.instance._workSpace.getCellHeight(),
                allDayHeight: this.instance._workSpace.getAllDayHeight()
            }
        }
    };
    _proto._getRenderingStrategyOptions = function() {
        var workspace = this.instance.getWorkSpace();
        var key = this.instance.key;
        var _this$instance$getWor = this.instance.getWorkSpace(),
            virtualScrollingDispatcher = _this$instance$getWor.virtualScrollingDispatcher;
        var cellCountInsideLeftVirtualCell = virtualScrollingDispatcher.cellCountInsideLeftVirtualCell,
            cellCountInsideTopVirtualRow = virtualScrollingDispatcher.cellCountInsideTopVirtualRow;
        return {
            instance: this.instance,
            key: key,
            isRenovatedAppointments: this.modelProvider.isRenovatedAppointments,
            viewRenderingStrategyName: this.viewRenderingStrategyName,
            adaptivityEnabled: this.modelProvider.adaptivityEnabled,
            rtlEnabled: this.modelProvider.rtlEnabled,
            startDayHour: this.modelProvider.startDayHour,
            endDayHour: this.modelProvider.endDayHour,
            maxAppointmentsPerCell: this.modelProvider.maxAppointmentsPerCell,
            agendaDuration: workspace.option("agendaDuration"),
            currentDate: this.modelProvider.currentDate,
            isVirtualScrolling: this.instance.isVirtualScrolling(),
            leftVirtualCellCount: cellCountInsideLeftVirtualCell,
            topVirtualCellCount: cellCountInsideTopVirtualRow,
            modelGroups: this.modelProvider.getCurrentViewOption("groups"),
            dateTableOffset: this.instance.getWorkSpaceDateTableOffset(),
            startViewDate: workspace.getStartViewDate(),
            groupOrientation: workspace._getRealGroupOrientation(),
            getIsGroupedByDate: function() {
                return workspace.isGroupedByDate()
            },
            getCellWidth: function() {
                return workspace.getCellWidth()
            },
            getCellHeight: function() {
                return workspace.getCellHeight()
            },
            getAllDayHeight: function() {
                return workspace.getAllDayHeight()
            },
            getResizableStep: function() {
                return workspace.positionHelper.getResizableStep()
            },
            getVisibleDayDuration: function() {
                return workspace.getVisibleDayDuration()
            },
            timeZoneCalculator: (0, _instanceFactory.getTimeZoneCalculator)(key),
            resourceManager: (0, _instanceFactory.getResourceManager)(key),
            appointmentDataProvider: (0, _instanceFactory.getAppointmentDataProvider)(key),
            timeZone: this.modelProvider.timeZone,
            firstDayOfWeek: this.instance.getFirstDayOfWeek(),
            viewStartDayHour: this.modelProvider.getCurrentViewOption("startDayHour"),
            viewEndDayHour: this.modelProvider.getCurrentViewOption("endDayHour"),
            viewType: workspace.type,
            endViewDate: workspace.getEndViewDate(),
            positionHelper: workspace.positionHelper,
            isGroupedByDate: workspace.isGroupedByDate(),
            cellDuration: workspace.getCellDuration(),
            viewDataProvider: workspace.viewDataProvider,
            supportAllDayRow: workspace.supportAllDayRow(),
            dateRange: workspace.getDateRange(),
            intervalDuration: workspace.getIntervalDuration(),
            isVerticalOrientation: workspace.isVerticalOrientation(),
            allDayIntervalDuration: workspace.getIntervalDuration(true),
            isSkippedDataCallback: workspace._isSkippedData.bind(workspace),
            getPositionShiftCallback: workspace.getPositionShift.bind(workspace),
            getGroupWidthCallback: workspace.getGroupWidth.bind(workspace),
            DOMMetaData: workspace.getDOMElementsMetaData()
        }
    };
    _proto.createAppointmentsMap = function(items) {
        var renderingStrategyOptions = this._getRenderingStrategyOptions();
        var _this$appointmentView = this.appointmentViewModel.generate(_extends({
                filteredItems: items
            }, renderingStrategyOptions)),
            viewModel = _this$appointmentView.viewModel,
            positionMap = _this$appointmentView.positionMap;
        this._positionMap = positionMap;
        return viewModel
    };
    _proto._isDataChanged = function(data) {
        var appointmentDataProvider = this.instance.fire("getAppointmentDataProvider");
        var updatedData = appointmentDataProvider.getUpdatedAppointment();
        return updatedData === data || appointmentDataProvider.getUpdatedAppointmentKeys().some((function(item) {
            return data[item.key] === item.value
        }))
    };
    _proto._isAppointmentShouldAppear = function(currentAppointment, sourceAppointment) {
        return currentAppointment.needRepaint && sourceAppointment.needRemove
    };
    _proto._isSettingChanged = function(settings, sourceSetting) {
        if (settings.length !== sourceSetting.length) {
            return true
        }
        var createSettingsToCompare = function(settings, index) {
            var currentSetting = settings[index];
            var leftVirtualCellCount = currentSetting.leftVirtualCellCount || 0;
            var topVirtualCellCount = currentSetting.topVirtualCellCount || 0;
            var columnIndex = currentSetting.columnIndex + leftVirtualCellCount;
            var rowIndex = currentSetting.rowIndex + topVirtualCellCount;
            var hMax = currentSetting.reduced ? currentSetting.hMax : void 0;
            var vMax = currentSetting.reduced ? currentSetting.vMax : void 0;
            return _extends({}, currentSetting, {
                columnIndex: columnIndex,
                rowIndex: rowIndex,
                topVirtualCellCount: void 0,
                leftVirtualCellCount: void 0,
                leftVirtualWidth: void 0,
                topVirtualHeight: void 0,
                hMax: hMax,
                vMax: vMax,
                info: {}
            })
        };
        for (var i = 0; i < settings.length; i++) {
            var newSettings = createSettingsToCompare(settings, i);
            var oldSettings = createSettingsToCompare(sourceSetting, i);
            if (oldSettings) {
                oldSettings.sortedIndex = newSettings.sortedIndex
            }
            if (!(0, _common.equalByValue)(newSettings, oldSettings)) {
                return true
            }
        }
        return false
    };
    _proto._getAssociatedSourceAppointment = function(currentAppointment, sourceAppointments) {
        for (var i = 0; i < sourceAppointments.length; i++) {
            var item = sourceAppointments[i];
            if (item.itemData === currentAppointment.itemData) {
                return item
            }
        }
        return null
    };
    _proto._getDeletedAppointments = function(currentAppointments, sourceAppointments) {
        var result = [];
        for (var i = 0; i < sourceAppointments.length; i++) {
            var sourceAppointment = sourceAppointments[i];
            var currentAppointment = this._getAssociatedSourceAppointment(sourceAppointment, currentAppointments);
            if (!currentAppointment) {
                sourceAppointment.needRemove = true;
                result.push(sourceAppointment)
            }
        }
        return result
    };
    _proto.getRepaintedAppointments = function(currentAppointments, sourceAppointments) {
        var _this = this;
        if (0 === sourceAppointments.length || "agenda" === this.viewRenderingStrategyName) {
            return currentAppointments
        }
        currentAppointments.forEach((function(appointment) {
            var sourceAppointment = _this._getAssociatedSourceAppointment(appointment, sourceAppointments);
            if (sourceAppointment) {
                appointment.needRepaint = _this._isDataChanged(appointment.itemData) || _this._isSettingChanged(appointment.settings, sourceAppointment.settings) || _this._isAppointmentShouldAppear(appointment, sourceAppointment)
            }
        }));
        return currentAppointments.concat(this._getDeletedAppointments(currentAppointments, sourceAppointments))
    };
    _proto.getRenderingStrategyInstance = function() {
        var renderingStrategy = this.appointmentViewModel.getRenderingStrategy();
        if (!renderingStrategy) {
            var options = this._getRenderingStrategyOptions();
            this.appointmentViewModel.initRenderingStrategy(options)
        }
        return this.appointmentViewModel.getRenderingStrategy()
    };
    _createClass(AppointmentLayoutManager, [{
        key: "modelProvider",
        get: function() {
            return (0, _instanceFactory.getModelProvider)(this.instance.key)
        }
    }, {
        key: "viewRenderingStrategyName",
        get: function() {
            return this.modelProvider.getViewRenderingStrategyName()
        }
    }]);
    return AppointmentLayoutManager
}();
var _default = AppointmentLayoutManager;
exports.default = _default;
module.exports = exports.default;
module.exports.default = exports.default;
