/**
 * DevExtreme (cjs/ui/scheduler/utils.js)
 * Version: 21.2.0 (build 21230-1556)
 * Build date: Wed Aug 18 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.utils = void 0;
var _renderer = _interopRequireDefault(require("../../core/renderer"));
var _extend = require("../../core/utils/extend");
var _iterator = require("../../core/utils/iterator");
var _instanceFactory = require("./instanceFactory");
var _constants = require("./constants");
var _element = require("../../core/element");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}
var utils = {
    dataAccessors: {
        getAppointmentSettings: function(element) {
            return (0, _renderer.default)(element).data(_constants.APPOINTMENT_SETTINGS_KEY)
        },
        getAppointmentInfo: function(element) {
            var settings = utils.dataAccessors.getAppointmentSettings(element);
            return null === settings || void 0 === settings ? void 0 : settings.info
        },
        combine: function(key, dataAccessors) {
            var result = (0, _extend.extend)(true, {}, dataAccessors);
            var resourceManager = (0, _instanceFactory.getResourceManager)(key);
            if (dataAccessors && resourceManager) {
                (0, _iterator.each)(resourceManager._dataAccessors, (function(type, accessor) {
                    result[type].resources = accessor
                }))
            }
            return result
        }
    },
    DOM: {
        getHeaderHeight: function(header) {
            return header._$element && parseInt(header._$element.outerHeight(), 10)
        }
    },
    renovation: {
        renderComponent: function(widget, parentElement, componentClass, componentName, viewModel) {
            var component = widget[componentName];
            if (!component) {
                var container = (0, _element.getPublicElement)(parentElement);
                component = widget._createComponent(container, componentClass, viewModel);
                widget[componentName] = component
            } else {
                var $element = component.$element();
                var elementStyle = $element.get(0).style;
                var height = elementStyle.height;
                var width = elementStyle.width;
                component.option(viewModel);
                height && $element.height(height);
                width && $element.width(width)
            }
        }
    }
};
exports.utils = utils;
