/**
 * DevExtreme (esm/exporter/jspdf/pdf_utils.js)
 * Version: 21.2.0 (build 21230-1556)
 * Build date: Wed Aug 18 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    isDefined
} from "../../core/utils/type";

function round(value) {
    return Math.round(1e3 * value) / 1e3
}

function calculateTextHeight(doc, text) {
    return doc.getTextDimensions(text).h * doc.getLineHeightFactor()
}

function calculateRowHeight(doc, cells) {
    var rowHeight = 0;
    for (var cellIndex = 0; cellIndex < cells.length; cellIndex++) {
        var cellText = cells[cellIndex].text;
        if (isDefined(cellText)) {
            var cellHeight = calculateTextHeight(doc, cellText);
            if (rowHeight < cellHeight) {
                rowHeight = cellHeight
            }
        }
    }
    return rowHeight
}

function drawLine(doc, startX, startY, endX, endY) {
    doc.line(round(startX), round(startY), round(endX), round(endY))
}

function drawRect(doc, x, y, width, height, style) {
    if (isDefined(style)) {
        doc.rect(round(x), round(y), round(width), round(height), style)
    } else {
        doc.rect(round(x), round(y), round(width), round(height))
    }
}

function drawText(doc, text, x, y, options) {
    doc.text(text, round(x), round(y), options)
}
export {
    calculateRowHeight,
    drawLine,
    drawRect,
    drawText
};
