/**
 * DevExtreme (esm/renovation/ui/scheduler/header/header.js)
 * Version: 21.2.0 (build 21230-1556)
 * Build date: Wed Aug 18 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["agendaDuration", "currentDate", "currentView", "currentViewChange", "customizationFunction", "defaultCurrentView", "firstDayOfWeek", "intervalCount", "items", "max", "min", "onCurrentDateUpdate", "onCurrentViewUpdate", "useDropDownViewSwitcher", "useShortDateFormat", "views"];
import {
    createComponentVNode
} from "inferno";
import {
    BaseInfernoComponent
} from "@devextreme/vdom";
import devices from "../../../../core/devices";
import {
    Toolbar
} from "../../toolbar/toolbar";
import "../../../../ui/button_group";
import "../../../../ui/drop_down_button";
import dateUtils from "../../../../core/utils/date";
import {
    getCaption,
    getStep,
    getViewName,
    getNextIntervalDate
} from "../../../../ui/scheduler/header/utils";
import {
    formToolbarItem,
    formatViews
} from "./utils";
import {
    SchedulerProps
} from "../props";
var {
    trimTime: trimTime
} = dateUtils;
export var viewFunction = viewModel => createComponentVNode(2, Toolbar, {
    items: viewModel.items
});
export var SchedulerToolbarBaseProps = {
    currentDate: new Date,
    intervalCount: 1,
    firstDayOfWeek: 0,
    agendaDuration: 7,
    useShortDateFormat: !devices.real().generic || devices.isSimulator()
};
export var SchedulerToolbarProps = {
    currentDate: SchedulerToolbarBaseProps.currentDate,
    intervalCount: SchedulerToolbarBaseProps.intervalCount,
    firstDayOfWeek: SchedulerToolbarBaseProps.firstDayOfWeek,
    agendaDuration: SchedulerToolbarBaseProps.agendaDuration,
    useShortDateFormat: SchedulerToolbarBaseProps.useShortDateFormat,
    useDropDownViewSwitcher: SchedulerProps.useDropDownViewSwitcher,
    defaultCurrentView: SchedulerProps.currentView,
    currentViewChange: () => {}
};
import {
    convertRulesToOptions
} from "../../../../core/options/utils";
export default class SchedulerToolbar extends BaseInfernoComponent {
    constructor(props) {
        super(props);
        this.state = {
            cssClass: "dx-scheduler-header",
            currentView: void 0 !== this.props.currentView ? this.props.currentView : this.props.defaultCurrentView
        };
        this.setCurrentView = this.setCurrentView.bind(this);
        this.setCurrentDate = this.setCurrentDate.bind(this);
        this.getNextDate = this.getNextDate.bind(this);
        this.updateDateByDirection = this.updateDateByDirection.bind(this);
        this.isPreviousButtonDisabled = this.isPreviousButtonDisabled.bind(this);
        this.isNextButtonDisabled = this.isNextButtonDisabled.bind(this);
        this.showCalendar = this.showCalendar.bind(this)
    }
    get step() {
        return getStep(void 0 !== this.props.currentView ? this.props.currentView : this.state.currentView)
    }
    get caption() {
        var options = {
            step: this.step,
            intervalCount: this.props.intervalCount,
            firstDayOfWeek: this.props.firstDayOfWeek,
            agendaDuration: this.props.agendaDuration,
            date: this.props.currentDate
        };
        return getCaption(options, this.props.useShortDateFormat, this.props.customizationFunction)
    }
    get captionText() {
        return this.caption.text
    }
    get views() {
        return formatViews(this.props.views)
    }
    get selectedView() {
        return getViewName(void 0 !== this.props.currentView ? this.props.currentView : this.state.currentView)
    }
    setCurrentView(view) {
        if (view.name !== (void 0 !== this.props.currentView ? this.props.currentView : this.state.currentView)) {
            this.props.onCurrentViewUpdate(view.name)
        }
    }
    setCurrentDate(date) {
        if (date.getTime() !== this.props.currentDate.getTime()) {
            this.props.onCurrentDateUpdate(new Date(date))
        }
    }
    get intervalOptions() {
        return {
            step: this.step,
            intervalCount: this.props.intervalCount,
            firstDayOfWeek: this.props.firstDayOfWeek,
            agendaDuration: this.props.agendaDuration
        }
    }
    getNextDate(direction, initialDate) {
        var date = null !== initialDate && void 0 !== initialDate ? initialDate : this.props.currentDate;
        var options = _extends({}, this.intervalOptions, {
            date: date
        });
        return getNextIntervalDate(options, direction)
    }
    updateDateByDirection(direction) {
        var date = this.getNextDate(direction);
        this.setCurrentDate(date)
    }
    isPreviousButtonDisabled() {
        if (void 0 === this.props.min) {
            return false
        }
        var min = trimTime(new Date(this.props.min));
        var {
            startDate: startDate
        } = this.caption;
        var previousDate = this.getNextDate(-1, startDate);
        return previousDate < min
    }
    isNextButtonDisabled() {
        if (void 0 === this.props.max) {
            return false
        }
        var max = new Date(new Date(this.props.max).setHours(23, 59, 59));
        var {
            endDate: endDate
        } = this.caption;
        var nextDate = this.getNextDate(1, endDate);
        return nextDate > max
    }
    showCalendar() {}
    get items() {
        var options = {
            useDropDownViewSwitcher: this.props.useDropDownViewSwitcher,
            selectedView: this.selectedView,
            views: this.views,
            setCurrentView: view => this.setCurrentView(view),
            showCalendar: () => this.showCalendar(),
            captionText: this.captionText,
            updateDateByDirection: direction => this.updateDateByDirection(direction),
            isPreviousButtonDisabled: this.isPreviousButtonDisabled(),
            isNextButtonDisabled: this.isNextButtonDisabled()
        };
        return this.props.items.map(item => formToolbarItem(item, options))
    }
    get restAttributes() {
        var _this$props$currentVi = _extends({}, this.props, {
                currentView: void 0 !== this.props.currentView ? this.props.currentView : this.state.currentView
            }),
            restProps = _objectWithoutPropertiesLoose(_this$props$currentVi, _excluded);
        return restProps
    }
    render() {
        var props = this.props;
        return viewFunction({
            props: _extends({}, props, {
                currentView: void 0 !== this.props.currentView ? this.props.currentView : this.state.currentView
            }),
            cssClass: this.state.cssClass,
            step: this.step,
            caption: this.caption,
            captionText: this.captionText,
            views: this.views,
            selectedView: this.selectedView,
            setCurrentView: this.setCurrentView,
            setCurrentDate: this.setCurrentDate,
            intervalOptions: this.intervalOptions,
            getNextDate: this.getNextDate,
            updateDateByDirection: this.updateDateByDirection,
            isPreviousButtonDisabled: this.isPreviousButtonDisabled,
            isNextButtonDisabled: this.isNextButtonDisabled,
            showCalendar: this.showCalendar,
            items: this.items,
            restAttributes: this.restAttributes
        })
    }
}

function __processTwoWayProps(defaultProps) {
    var twoWayProps = ["currentView"];
    return Object.keys(defaultProps).reduce((props, propName) => {
        var propValue = defaultProps[propName];
        var defaultPropName = twoWayProps.some(p => p === propName) ? "default" + propName.charAt(0).toUpperCase() + propName.slice(1) : propName;
        props[defaultPropName] = propValue;
        return props
    }, {})
}

function __createDefaultProps() {
    return _extends({}, SchedulerToolbarProps)
}
SchedulerToolbar.defaultProps = __createDefaultProps();
var __defaultOptionRules = [];
export function defaultOptions(rule) {
    __defaultOptionRules.push(rule);
    SchedulerToolbar.defaultProps = _extends({}, __createDefaultProps(), __processTwoWayProps(convertRulesToOptions(__defaultOptionRules)))
}
